/* 1541control - way overdesigned LCD addon for a 1541
   Copyright (C) 2007  Ingo Korb <ingo@akana.de>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; version 2 of the License only.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   
   encoder.c: Rotary encoder status (and timekeeping)

*/

#include "config.h"
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include "encoder.h"
#include "tracker.h"

volatile jiffy_t jiffies;
volatile int8_t  encoder_position;
volatile jiffy_t last_encoder_change;
volatile uint8_t key_state;
volatile jiffy_t last_key_change;

uint8_t laststate;
jiffy_t debounce_timer;

ISR(TIMER1_COMPA_vect) {
  uint8_t curstate;

  jiffies++;

  /* Encoderbehandlung */
  curstate = PINB & (_BV(PB0) | _BV(PB1));

  // Encoder
  if ((laststate&1) ^ (curstate&1)) {
    if (!!(curstate&1) == !!(curstate&2)) {
      encoder_position--;
    } else {
      encoder_position++;
    }
    last_encoder_change = jiffies;
    laststate = curstate;
  }

  // Taster
  curstate = !(PINB & _BV(PB2));
  if (curstate != key_state) {
    if (last_key_change != debounce_timer) {
      if ((jiffies-debounce_timer) > 50) {
	last_key_change = jiffies;
	debounce_timer = jiffies;
	key_state = curstate;
      }
    } else {
      debounce_timer = jiffies;
    }
  } else {
    // Zurueckgebounct - Timer zuruecksetzen
    debounce_timer = last_key_change;
  }

  update_tracker();
}

void init_encoder(void) {
  DDRB  &= ~(_BV(PB0) | _BV(PB1) | _BV(PB2));
  PORTB |=   _BV(PB0) | _BV(PB1) | _BV(PB2);
  laststate = PINB & (_BV(PB0) | _BV(PB1));

  // Temporaerer fix fuer lange Leitung am Encoder waehrend der Entwicklung
  _delay_ms(20);

  jiffies = 0;
  encoder_position = 0;
  last_encoder_change = 0;
  key_state = !(PINB & _BV(PB2));
  last_key_change = 0;
  debounce_timer = 0;

  // Timer 1 als Interruptquelle, 1000/s
  TCCR1A = 0;
  TCCR1B = _BV(WGM12);
  TCNT1  = 0;
  OCR1A  = F_CPU/1000;
  TIMSK |= _BV(OCIE1A);
  
  // Timer starten
  TCCR1B |= _BV(CS10);
}
