/* 1541control - way overdesigned LCD addon for a 1541
   Copyright (C) 2007  Ingo Korb <ingo@akana.de>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; version 2 of the License only.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   
   menu.c: Generic menu code

*/

#include "config.h"
#include <string.h>
#include <avr/io.h>
#include "encoder.h"
#include "lcd.h"
#include "menu.h"
#include "uart.h"

/* Nicht seiteneffekt-sicher! */
#define min(x,y) ((x)<(y)?(x):(y))
#define max(x,y) ((x)>(y)?(x):(y))

static void show_menu(char **entries, uint8_t count, uint8_t start) {
  uint8_t i,j;

  lcd_gotoxy(1,0);
  for (i=0;i<min(count,LCD_ROWS);i++) {
    for (j=0;j<LCD_COLUMNS-1 && entries[i+start][j];j++)
      lcd_putch(entries[i+start][j]);
    for (;j<LCD_COLUMNS;j++)
      lcd_putch(' ');
  }
  for (;i<LCD_ROWS;i++)
    for (j=0;j<LCD_COLUMNS;j++)
      lcd_putch(' ');
}

void init_menu(void) {
  lcd_customchar(3,0x08,0x0c,0x0e,0x0f,0x0e,0x0c,0x08,0x00);
}

uint8_t display_menu(char **entries, uint8_t count, int8_t curentry) {
  int8_t  prevoffset,curoffset,preventry,scrollofs;
  int8_t  curencoder,prevencoder;
  uint8_t prevkey,curkey,entrylen;
  jiffy_t scrolltime;

  lcd_setcursormode(LCD_CURSOR_NONE);
  lcd_clrscr();

  prevencoder = encoder_position;
  prevkey     = 0;
  prevoffset  = -1;
  preventry   = curentry;
  scrollofs   = -1;
  scrolltime  = 0;
  entrylen    = strlen(entries[curentry]);
  
  curoffset = curentry-LCD_ROWS/2+1;
  if (curoffset+LCD_ROWS-1 >= count) curoffset = count-LCD_ROWS;
  if (curoffset < 0) curoffset = 0;

  while (1) {
    curencoder = encoder_position;
    curkey     = key_state;

    /* Tastendruck beendet Menu */
    if (prevkey && !curkey)
      return curentry;

    /* Encoderdrehung */
    if (curencoder != prevencoder) {
      curentry += (curencoder-prevencoder);
      if (curentry >= count) curentry = count-1;
      if (curentry < 0) curentry = 0;
    }
  
    /* Offset verschieben bis der Marker wieder in gewuenschtem Bereich ist */
    while (curentry-curoffset < 1 && curoffset > 0) curoffset--;
    while (curentry-curoffset >= LCD_ROWS-1 && curentry < count-1) curoffset++;

    /* Bei geaendertem Offset komplett neuzeichnen */
    if (curoffset != prevoffset) {
      show_menu(entries,count,curoffset);
      prevoffset = curoffset;
      scrollofs  = -1;

      lcd_gotoxy(0,curentry-curoffset);
      lcd_putch(3);
    } 

    /* Bei geaendertem Eintrag nur Marker verschieben */
    if (preventry != curentry) {
      lcd_gotoxy(0,preventry-curoffset);
      lcd_putch(' ');

      if (scrollofs != -1) {
	uint8_t i;

	lcd_gotoxy(1,preventry-curoffset);
	for (i=0;i<LCD_COLUMNS-1;i++)
	  lcd_putch(entries[preventry][i]);
	scrollofs = -1;
      }

      lcd_gotoxy(0,curentry-curoffset);
      lcd_putch(3);
      
      entrylen = strlen(entries[curentry]);

      preventry = curentry;
    }

    /* Aktueller Eintrag lang -> Scrollen */
    if (entrylen >= LCD_COLUMNS) {
      if (scrollofs == -1) {
	/* Noch nicht gescrollt, in 2s anfangen */
	scrollofs  = 0;
	scrolltime = jiffies + TICKS_BEFORE_SCROLLING;
      } else if (time_after(jiffies,scrolltime)) {
	/* Scrolling now! */
	uint8_t i;

	scrollofs++;
	if (scrollofs > entrylen+2) scrollofs = 0;
	scrolltime += TICKS_PER_CHARACTER;

	lcd_gotoxy(1,curentry-curoffset);
	for (i=0;i<LCD_COLUMNS-1;i++) {
	  if (scrollofs+i < entrylen) {
	    lcd_putch(entries[curentry][scrollofs+i]);
	  } else if (scrollofs+i < entrylen+3) {
	    lcd_putch(' ');
	  } else
	    lcd_putch(entries[curentry][scrollofs+i-entrylen-3]);
	}
      }
    }

    prevencoder = curencoder;
    prevkey     = curkey;
  }
}
