/* 1541control - way overdesigned LCD addon for a 1541
   Copyright (C) 2007  Ingo Korb <ingo@akana.de>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; version 2 of the License only.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   
   tracker.c: Stepper motor position estimator

*/

#include "config.h"
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <stdlib.h>
#include "encoder.h"
#include "tracker.h"

#include <stdio.h>
#include "uart.h"

volatile  int8_t current_track,current_density;
volatile uint8_t tracker_flags;

volatile uint8_t lasttrack,lastdensity;
volatile  int8_t mintrack,maxtrack;
jiffy_t lastreset;

//static const int8_t trackdiff[4][4] = {{0,1,2,-1},{-1,0,1,2},{2,-1,0,1},{1,2,-1,0}};
static const int8_t trackdiff[4][4] = {{0,2,1,-1},{2,0,-1,1},{-1,1,0,2},{1,-1,2,0}};

#define MAX_TRACK (43*2+1)

void update_tracker(void) {
  uint8_t curstate,savedstate;
  int8_t tmp;

  // Floppyreset abfragen
  /*
  if (!(TRACK_PIN & _BV(TRACK_RESET))) {
    mintrack = current_track - 3;
    maxtrack = current_track + 3;
    tracker_flags = TRACK_CHANGED;
  }
  */

  curstate = TRACK_PIN & (_BV(TRACK_BIT0) | _BV(TRACK_BIT1));
  if (curstate != lasttrack) {
    savedstate = curstate;
    curstate  >>= TRACK_BIT0;
    lasttrack >>= TRACK_BIT0;
    //tmp = curstate-lasttrack;
    tmp = trackdiff[lasttrack][curstate];
    if (tmp == 2) {
      // Mehr als einen Schritt gemacht
      tracker_flags = TRACK_CHANGED;
    } else if (tracker_flags & TRACK_VALID) {
      // Schon gltiger Track bekannt
      current_track += tmp;
      // Bump
      if (current_track < 2) current_track = 2;
      if (current_track > MAX_TRACK) current_track = MAX_TRACK;
      tracker_flags = TRACK_VALID | TRACK_CHANGED;
      mintrack = current_track;
      maxtrack = current_track;
    } else {
      // Track noch nicht gefunden, Grenzen schieben
      if (tmp < 0) {
	maxtrack--;
	mintrack--;
	if (maxtrack < 2) maxtrack = 2;
	if (mintrack < 2) mintrack = 2;
      } else {
	mintrack++;
	maxtrack++;
	if (mintrack > MAX_TRACK) mintrack = MAX_TRACK;
	if (maxtrack > MAX_TRACK) maxtrack = MAX_TRACK;
      }
    }
    
    if (mintrack == maxtrack) {
      // Track gefunden, gltig-Flag setzen
      current_track = mintrack;
      tracker_flags = TRACK_VALID | TRACK_CHANGED;
    }
    lasttrack = savedstate;
  }

  // Density-Pins abfragen
  curstate = (DENSITY_PIN & (_BV(DENSITY_BIT0) | _BV(DENSITY_BIT1))) >> DENSITY_BIT0;
  if (curstate != lastdensity) {
    current_density = 3-curstate;
    lastdensity = curstate;
    tracker_flags |= DENSITY_CHANGED;
  }
}

void init_tracker(void) {
  // Eingnge fr Steppermotorsignale und MOTOR_ON
  TRACK_DDR  &= ~(_BV(TRACK_BIT0) | _BV(TRACK_BIT1) | _BV(TRACK_MOTOR));
  TRACK_PORT &= ~(_BV(TRACK_BIT0) | _BV(TRACK_BIT1) | _BV(TRACK_MOTOR));

  // Eingnge fr Density-Signale
  DENSITY_DDR  &= ~(_BV(DENSITY_BIT0) | _BV(DENSITY_BIT1));
  DENSITY_PORT &= ~(_BV(DENSITY_BIT0) | _BV(DENSITY_BIT1));
  
  current_track   = -1;
  current_density = (DENSITY_PIN & (_BV(DENSITY_BIT0) | _BV(DENSITY_BIT1))) >> DENSITY_BIT0;
  mintrack        = 1;
  maxtrack        = MAX_TRACK;
  tracker_flags   = 0;
  lasttrack       = TRACK_PIN & (_BV(TRACK_BIT0)|_BV(TRACK_BIT1));
  lastdensity     = current_density;
  lastreset       = 0;
}
